=== renamed directory 'ubuntu-emulator/diskimage' => 'diskimage'
=== modified file 'diskimage/image.go'
--- ubuntu-emulator/diskimage/image.go	2014-07-02 18:45:34 +0000
+++ diskimage/image.go	2014-10-09 12:18:37 +0000
@@ -28,13 +28,36 @@
 	"os"
 	"os/exec"
 	"path/filepath"
-
-	"launchpad.net/goget-ubuntu-touch/ubuntu-emulator/sysutils"
-)
+	"strings"
+
+	"launchpad.net/goget-ubuntu-touch/sysutils"
+)
+
+type imageLabel string
+type directory string
+
+const (
+	systemDataLabel imageLabel = "system-data"
+	userDataLabel   imageLabel = "user-data"
+)
+
+const (
+	systemDataDir directory = "system"
+	userDataDir   directory = "user-data"
+)
+
+type partition struct {
+	label imageLabel
+	dir   directory
+	loop  string
+}
 
 type DiskImage struct {
-	path, label, Mountpoint string
-	size                    int64
+	Mountpoint string
+	label      string
+	path       string
+	size       int64
+	parts      []partition
 }
 
 //New creates a new DiskImage
@@ -91,6 +114,32 @@
 	return nil
 }
 
+// User returns the user path
+func (img *DiskImage) User() (string, error) {
+	if img.parts == nil {
+		return "", errors.New("img is not setup with partitions")
+	}
+
+	if img.Mountpoint == "" {
+		return "", errors.New("img not mounted")
+	}
+
+	return filepath.Join(img.Mountpoint, string(userDataDir)), nil
+}
+
+//System returns the system path
+func (img *DiskImage) System() (string, error) {
+	if img.parts == nil {
+		return "", errors.New("img is not setup with partitions")
+	}
+
+	if img.Mountpoint == "" {
+		return "", errors.New("img not mounted")
+	}
+
+	return filepath.Join(img.Mountpoint, string(systemDataDir)), nil
+}
+
 //Mount the DiskImage
 func (img *DiskImage) Mount() (err error) {
 	img.Mountpoint, err = ioutil.TempDir(os.TempDir(), "ubuntu-system")
@@ -103,8 +152,29 @@
 			os.Remove(img.Mountpoint)
 		}
 	}()
+	if img.parts == nil {
+		return img.emulatorMount()
+	}
+
+	return img.coreMount()
+}
+
+func (img *DiskImage) coreMount() (err error) {
+	for _, part := range img.parts {
+		mountpoint := filepath.Join(img.Mountpoint, string(part.dir))
+		if err := os.MkdirAll(mountpoint, 0755); err != nil {
+			return err
+		}
+		if out, err := exec.Command("mount", filepath.Join("/dev/mapper", part.loop), mountpoint).CombinedOutput(); err != nil {
+			return fmt.Errorf("unable to dir to create system image: %s", out)
+		}
+	}
+	return nil
+}
+
+func (img *DiskImage) emulatorMount() (err error) {
 	if out, err := exec.Command("mount", img.path, img.Mountpoint).CombinedOutput(); err != nil {
-		return errors.New(fmt.Sprintf("Unable to mount temp dir to create system image %s", out))
+		return fmt.Errorf("unable to mount temp dir to create system image: %s", out)
 	}
 	return nil
 }
@@ -116,13 +186,41 @@
 	}
 	defer os.Remove(img.Mountpoint)
 	if out, err := exec.Command("sync").CombinedOutput(); err != nil {
-		exec.Command("umount", img.Mountpoint).Run()
 		return errors.New(fmt.Sprintf("Failed to sync filesystems before unmounting: %s", out))
 	}
+
+	if img.parts == nil {
+		if err := img.emulatorUnmount(); err != nil {
+			return err
+		}
+	} else {
+		if err := img.coreUnmount(); err != nil {
+			return err
+		}
+	}
+
+	img.Mountpoint = ""
+
+	return nil
+}
+
+func (img *DiskImage) coreUnmount() (err error) {
+	for _, part := range img.parts {
+		mountpoint := filepath.Join(img.Mountpoint, string(part.dir))
+		if out, err := exec.Command("umount", mountpoint).CombinedOutput(); err != nil {
+			return fmt.Errorf("unable to dir to create system image: %s", out)
+		}
+		if err := os.Remove(mountpoint); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+func (img *DiskImage) emulatorUnmount() (err error) {
 	if err := exec.Command("umount", img.Mountpoint).Run(); err != nil {
 		return errors.New("Failed to unmount temp dir where system image was created")
 	}
-	img.Mountpoint = ""
 	return nil
 }
 
@@ -146,15 +244,100 @@
 	return nil
 }
 
-//Create returns a ext4 partition for a given file
+//Partition creates a partitioned image from an img
+func (img *DiskImage) Partition() error {
+	if err := sysutils.CreateEmptyFile(img.path, img.size); err != nil {
+		return err
+	}
+
+	partedCmd := exec.Command("parted", img.path)
+	stdin, err := partedCmd.StdinPipe()
+	if err != nil {
+		return err
+	}
+
+	stdin.Write([]byte("mklabel msdos\n"))
+	stdin.Write([]byte("mkpart primary ext4 2048s 4G\n"))
+	stdin.Write([]byte("mkpart primary ext4 4G -1s\n"))
+	stdin.Write([]byte("set 1 boot on\n"))
+	stdin.Write([]byte("print\n"))
+	stdin.Write([]byte("quit\n"))
+
+	return partedCmd.Run()
+}
+
+//MapPartitions creates loop devices for the partitions
+func (img *DiskImage) MapPartitions() error {
+	kpartxCmd := exec.Command("kpartx", "-avs", img.path)
+	stdout, err := kpartxCmd.StdoutPipe()
+	if err != nil {
+		return err
+	}
+
+	if err := kpartxCmd.Start(); err != nil {
+		return err
+	}
+
+	loops := make([]string, 0, 2)
+	scanner := bufio.NewScanner(stdout)
+	for scanner.Scan() {
+		fields := strings.Fields(scanner.Text())
+		if len(fields) > 2 {
+			loops = append(loops, fields[2])
+		} else {
+			return errors.New("issues while determining drive mappings")
+		}
+	}
+	if err := scanner.Err(); err != nil {
+		return err
+	}
+
+	if len(loops) != 2 {
+		return errors.New("more partitions then expected while creating loop mapping")
+	}
+
+	img.parts = []partition{
+		partition{label: systemDataLabel, dir: systemDataDir, loop: loops[0]},
+		partition{label: userDataLabel, dir: userDataDir, loop: loops[1]},
+	}
+
+	if err := kpartxCmd.Wait(); err != nil {
+		return err
+	}
+
+	return nil
+}
+
+//UnMapPartitions destroys loop devices for the partitions
+func (img *DiskImage) UnMapPartitions() error {
+	for _, part := range img.parts {
+		if err := exec.Command("dmsetup", "clear", part.loop).Run(); err != nil {
+			return err
+		}
+	}
+	return exec.Command("kpartx", "-d", img.path).Run()
+}
+
+//CreateExt4 returns a ext4 partition for a given file
 func (img DiskImage) CreateExt4() error {
-	if err := sysutils.CreateEmptyFile(img.path, img.size); err != nil {
-		return err
-	}
-	return exec.Command("mkfs.ext4", "-F", "-L", img.label, img.path).Run()
+	if img.parts == nil {
+		if err := sysutils.CreateEmptyFile(img.path, img.size); err != nil {
+			return err
+		}
+		return exec.Command("mkfs.ext4", "-F", "-L", img.label, img.path).Run()
+	}
+
+	for _, part := range img.parts {
+		dev := filepath.Join("/dev/mapper", part.loop)
+		if err := exec.Command("mkfs.ext4", "-F", "-L", string(part.label), dev).Run(); err != nil {
+			return err
+		}
+	}
+
+	return nil
 }
 
-//Create returns a vfat partition for a given file
+//CreateVFat returns a vfat partition for a given file
 func (img DiskImage) CreateVFat() error {
 	if err := sysutils.CreateEmptyFile(img.path, img.size); err != nil {
 		return err

=== renamed directory 'ubuntu-emulator/sysutils' => 'sysutils'
=== modified file 'ubuntu-device-flash/args.go'
--- ubuntu-device-flash/args.go	2014-09-30 13:11:28 +0000
+++ ubuntu-device-flash/args.go	2014-10-09 12:18:37 +0000
@@ -20,39 +20,16 @@
 // You should have received a copy of the GNU General Public License along
 // with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import (
-	flags "github.com/jessevdk/go-flags"
-)
+import flags "github.com/jessevdk/go-flags"
 
 type arguments struct {
-	Revision      int    `long:"revision" description:"revision to flash, absolute or relative allowed"`
-	Bootstrap     bool   `long:"bootstrap" description:"bootstrap the system, do this from the bootloader"`
-	ListChannels  bool   `long:"list-channels" description:"List available channels"`
-	ListImages    bool   `long:"list-images" description:"List available images for a channel"`
-	Wipe          bool   `long:"wipe" description:"Clear all data after flashing"`
-	Channel       string `long:"channel" description:"Specify an alternate channel"`
-	ShowImage     bool   `long:"show-image" description:"Show information for an image in the given channel"`
-	Device        string `long:"device" description:"Specify the device to flash"`
-	DeviceTarball string `long:"device-tarball" description:"Specify a local device tarball to override the one from the server (using official Ubuntu images with custom device tarballs)"`
-	DownloadOnly  bool   `long:"download-only" description:"Only download tarballs, do not push to the device."`
-	Serial        string `long:"serial" description:"Serial of the device to operate"`
-	Server        string `long:"server" description:"Use a different image server"`
+	Revision      int    `long:"revision" description:"revision to use, absolute or relative allowed"`
+	Channel       string `long:"channel" description:"(DEPRECATED) Specify the channel to use" default:"ubuntu-touch/stable"`
+	DownloadOnly  bool   `long:"download-only" description:"Only download."`
+	Server        string `long:"server" description:"Use a different image server" default:"https://system-image.ubuntu.com"`
 	CleanCache    bool   `long:"clean-cache" description:"Cleans up cache with all downloaded bits"`
 	TLSSkipVerify bool   `long:"tls-skip-verify" description:"Skip TLS certificate validation"`
-	DeveloperMode bool   `long:"developer-mode" description:"Enables developer mode after the factory reset"`
-	RunScript     string `long:"run-script" description:"Run a script given by path to finish the flashing process, instead of rebooting to recovery (mostly used during development to work around quirky or incomplete recovery images)"`
-	Password      string `long:"password" description:"This sets up the default password for the phablet user. This option is meant for CI and not general use"`
-}
-
-var args arguments
-var parser = flags.NewParser(&args, flags.Default)
-
-const (
-	defaultChannel = "ubuntu-touch/stable"
-	defaultServer  = "https://system-image.ubuntu.com"
-)
-
-func init() {
-	args.Channel = defaultChannel
-	args.Server = defaultServer
-}
+}
+
+var globalArgs arguments
+var parser = flags.NewParser(&globalArgs, flags.Default)

=== added file 'ubuntu-device-flash/common.go'
--- ubuntu-device-flash/common.go	1970-01-01 00:00:00 +0000
+++ ubuntu-device-flash/common.go	2014-10-09 12:18:37 +0000
@@ -0,0 +1,33 @@
+package main
+
+import (
+	"log"
+	"path/filepath"
+
+	"launchpad.net/goget-ubuntu-touch/ubuntuimage"
+)
+
+func getImage(deviceChannel ubuntuimage.DeviceChannel) (image ubuntuimage.Image, err error) {
+	if globalArgs.Revision <= 0 {
+		image, err = deviceChannel.GetRelativeImage(globalArgs.Revision)
+	} else {
+		image, err = deviceChannel.GetImage(globalArgs.Revision)
+	}
+	return image, err
+}
+
+type Files struct{ FilePath, SigPath string }
+
+// bitDownloader downloads
+func bitDownloader(file ubuntuimage.File, files chan<- Files, server, downloadDir string) {
+	err := file.MakeRelativeToServer(server)
+	if err != nil {
+		log.Fatal(err)
+	}
+	err = file.Download(downloadDir)
+	if err != nil {
+		log.Fatal(err)
+	}
+	files <- Files{FilePath: filepath.Join(downloadDir, file.Path),
+		SigPath: filepath.Join(downloadDir, file.Signature)}
+}

=== added file 'ubuntu-device-flash/core.go'
--- ubuntu-device-flash/core.go	1970-01-01 00:00:00 +0000
+++ ubuntu-device-flash/core.go	2014-10-09 12:18:37 +0000
@@ -0,0 +1,258 @@
+//
+// udbflash - Tool to download and flash devices with an Ubuntu Image based
+//            system
+//
+// Copyright (c) 2013 Canonical Ltd.
+//
+// Written by Sergio Schvezov <sergio.schvezov@canonical.com>
+//
+package main
+
+import (
+	"errors"
+	"fmt"
+	"os"
+	"os/exec"
+	"path/filepath"
+	"runtime"
+	"syscall"
+	"time"
+
+	"launchpad.net/goget-ubuntu-touch/diskimage"
+	"launchpad.net/goget-ubuntu-touch/sysutils"
+	"launchpad.net/goget-ubuntu-touch/ubuntuimage"
+)
+
+// This program is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License version 3, as published
+// by the Free Software Foundation.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranties of
+// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
+// PURPOSE.  See the GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+func init() {
+	parser.AddCommand("core",
+		"Creates ubuntu core images",
+		"",
+		&coreCmd)
+}
+
+type CoreCmd struct {
+	Channel string `long:"channel" description:"Specify the channel to use" default:"ubuntu-core/utopic"`
+	Device  string `long:"device" description:"Specify the device to use" default:"generic_amd64"`
+	Output  string `long:"output" short:"o" description:"Name of the image file to create" required:"true"`
+	Size    int64  `long:"size" short:"s" description:"Size of image file to create in GB (min 6)" default:"20"`
+}
+
+var coreCmd CoreCmd
+
+func (coreCmd *CoreCmd) Execute(args []string) error {
+	if syscall.Getuid() != 0 {
+		return errors.New("command requires sudo/pkexec (root)")
+	}
+
+	// hack to circumvent https://code.google.com/p/go/issues/detail?id=1435
+	runtime.GOMAXPROCS(1)
+	runtime.LockOSThread()
+	if err := sysutils.DropPrivs(); err != nil {
+		return err
+	}
+
+	channels, err := ubuntuimage.NewChannels(globalArgs.Server)
+	if err != nil {
+		return err
+	}
+
+	deviceChannel, err := channels.GetDeviceChannel(globalArgs.Server, coreCmd.Channel, coreCmd.Device)
+	if err != nil {
+		return err
+	}
+
+	image, err := getImage(deviceChannel)
+	if err != nil {
+		return err
+	}
+
+	filesChan := make(chan Files, len(image.Files))
+	defer close(filesChan)
+
+	sigFiles := ubuntuimage.GetGPGFiles()
+	sigFilesChan := make(chan Files, len(sigFiles))
+	defer close(sigFilesChan)
+
+	for _, f := range sigFiles {
+		go bitDownloader(f, sigFilesChan, globalArgs.Server, cacheDir)
+	}
+
+	for _, f := range image.Files {
+		go bitDownloader(f, filesChan, globalArgs.Server, cacheDir)
+	}
+
+	filePathChan := make(chan string)
+
+	go func() {
+		for i := 0; i < len(image.Files); i++ {
+			f := <-filesChan
+			fmt.Println("Download finished for", f.FilePath)
+			filePathChan <- f.FilePath
+		}
+		close(filePathChan)
+	}()
+
+	img := diskimage.New(coreCmd.Output, "", coreCmd.Size)
+	if err := img.Partition(); err != nil {
+		return err
+	}
+	defer func() {
+		if err != nil {
+			os.Remove(coreCmd.Output)
+		}
+	}()
+
+	{
+		if err := sysutils.EscalatePrivs(); err != nil {
+			return err
+		}
+		defer sysutils.DropPrivs()
+
+		if err := coreCmd.partition(img); err != nil {
+			return err
+		}
+
+		if err := coreCmd.setup(img, filePathChan); err != nil {
+			return err
+		}
+	}
+
+	fmt.Println("New image complete, launch by running: kvm", coreCmd.Output)
+
+	return nil
+}
+
+func (coreCmd *CoreCmd) partition(img *diskimage.DiskImage) error {
+	if err := img.MapPartitions(); err != nil {
+		return err
+	}
+	defer img.UnMapPartitions()
+
+	return img.CreateExt4()
+}
+
+func (coreCmd *CoreCmd) setup(img *diskimage.DiskImage, filePathChan <-chan string) error {
+	if err := img.MapPartitions(); err != nil {
+		return err
+	}
+	defer img.UnMapPartitions()
+
+	if err := img.Mount(); err != nil {
+		return err
+	}
+	defer img.Unmount()
+
+	for f := range filePathChan {
+		if out, err := exec.Command("tar", "-axvf", "--numeric-owner", f, "-C", img.Mountpoint).CombinedOutput(); err != nil {
+			return fmt.Errorf("issues while extracting: %s", out)
+		}
+	}
+
+	{
+		systemPath, err := img.System()
+		if err != nil {
+			return err
+		}
+
+		if err := coreCmd.setupBootloader(systemPath); err != nil {
+			return err
+		}
+	}
+
+	{
+		userPath, err := img.User()
+		if err != nil {
+			return err
+		}
+
+		for _, dir := range []string{"system-data", "user-data", "cache"} {
+			dirPath := filepath.Join(userPath, dir)
+			if err := os.Mkdir(dirPath, 0755); err != nil {
+				return err
+			}
+		}
+	}
+
+	return nil
+}
+
+func (coreCmd *CoreCmd) setupBootloader(systemPath string) error {
+	for _, dev := range []string{"dev", "proc", "sys"} {
+		src := filepath.Join("/", dev)
+		dst := filepath.Join(systemPath, dev)
+		if err := bindMount(src, dst); err != nil {
+			return err
+		}
+		defer unmount(dst)
+	}
+
+	firmwarePath := filepath.Join(systemPath, "sys", "firmware")
+	if err := bindMount(filepath.Join(systemPath, "mnt"), firmwarePath); err != nil {
+		return err
+	}
+	defer unmount(firmwarePath)
+
+	outputPath, err := filepath.Abs(coreCmd.Output)
+	if err != nil {
+		return errors.New("cannot determined absolute path for output image")
+	}
+
+	rootDevPath := filepath.Join(systemPath, "root_dev")
+
+	if f, err := os.Create(rootDevPath); err != nil {
+		return err
+	} else {
+		f.Close()
+		defer os.Remove(rootDevPath)
+	}
+
+	if err := bindMount(outputPath, rootDevPath); err != nil {
+		return err
+	}
+	defer unmount(rootDevPath)
+
+	if out, err := exec.Command("chroot", systemPath, "grub-install", "/root_dev").CombinedOutput(); err != nil {
+		return fmt.Errorf("unable to install grub: %s", out)
+	} else {
+		fmt.Println(string(out))
+	}
+
+	// I don't know why this is needed, I just picked it up from the original implementation
+	time.Sleep(3 * time.Second)
+
+	if out, err := exec.Command("chroot", systemPath, "update-grub").CombinedOutput(); err != nil {
+		return fmt.Errorf("unable to update grub: %s", out)
+	} else {
+		fmt.Println(string(out))
+	}
+
+	return nil
+}
+
+func bindMount(src, dst string) error {
+	if out, err := exec.Command("mount", "--bind", src, dst).CombinedOutput(); err != nil {
+		return fmt.Errorf("issues while bind mounting: %s", out)
+	}
+
+	return nil
+}
+
+func unmount(dst string) error {
+	if out, err := exec.Command("umount", dst).CombinedOutput(); err != nil {
+		return fmt.Errorf("issues while unmounting: %s", out)
+	}
+
+	return nil
+}

=== modified file 'ubuntu-device-flash/main.go'
--- ubuntu-device-flash/main.go	2014-10-02 08:48:42 +0000
+++ ubuntu-device-flash/main.go	2014-10-09 12:18:37 +0000
@@ -21,406 +21,28 @@
 // with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import (
-	"archive/tar"
-	"errors"
 	"fmt"
-	"io"
-	"io/ioutil"
-	"log"
 	"os"
-	"os/exec"
-	"path/filepath"
-	"strings"
-	"syscall"
 
-	"launchpad.net/goget-ubuntu-touch/devices"
 	"launchpad.net/goget-ubuntu-touch/ubuntuimage"
 )
 
+var cacheDir = ubuntuimage.GetCacheDir()
+
 func main() {
-	if _, err := parser.Parse(); err != nil {
+	args := os.Args
+
+	parser.SubcommandsOptional = true
+	if _, err := parser.ParseArgs(args); err != nil {
 		os.Exit(1)
 	}
-	if args.TLSSkipVerify {
-		ubuntuimage.TLSSkipVerify()
-	}
-	script := args.RunScript
-	if script != "" {
-		if p, err := filepath.Abs(script); err != nil {
-			log.Fatal("Run script not found:", err)
-		} else {
-			script = p
-		}
-
-		fi, err := os.Lstat(script)
-		if err != nil {
-			log.Fatal(err)
-		}
-		if fi.Mode()&0100 == 0 || !fi.Mode().IsRegular() {
-			log.Fatalf("The script %s passed via --run-script is not executable", script)
-		}
-	}
-
-	if args.Bootstrap {
-		args.Wipe = true
-	}
-
-	if args.Password != "" && !args.Wipe && !args.DeveloperMode {
-		log.Fatal("Default password setup requires --developer-mode, and --wipe or --bootstrap")
-	}
-
-	tarballPath := args.DeviceTarball
-	if tarballPath != "" {
-		if p, err := filepath.Abs(tarballPath); err != nil {
-			log.Fatal("Device tarball not found", err)
-		} else {
-			tarballPath = p
-		}
-		fi, err := os.Lstat(tarballPath)
-		if err != nil {
-			log.Fatal(err)
-		}
-		if !fi.Mode().IsRegular() {
-			log.Fatalf("The file %s passed via --device-tarball is not a regular file\n", tarballPath)
-		}
-	}
-	channels, err := ubuntuimage.NewChannels(args.Server)
-	if err != nil {
-		log.Fatal(err)
-	}
-	if args.ListChannels {
-		for k, v := range channels {
-			if v.Alias != "" {
-				fmt.Printf("%s (alias to %s)\n", k, v.Alias)
-			} else {
-				fmt.Println(k)
-			}
-		}
-		return
-	}
-
-	if args.ListImages {
-		if args.Device == "" {
-			log.Fatal("You must specify a device type")
-		}
-
-		deviceChannel, err := channels.GetDeviceChannel(
-			args.Server, args.Channel, args.Device)
-		if err != nil {
-			log.Fatal(err)
-		}
-
-		err = deviceChannel.ListImageVersions()
-		if err != nil {
-			log.Fatal(err)
-		}
-		return
-	}
-
-	cacheDir := ubuntuimage.GetCacheDir()
-	if args.CleanCache {
-		log.Print("Cleaning prevously downloaded content")
-		if err := os.RemoveAll(cacheDir); err != nil {
-			log.Fatal(err)
-		}
-		return
-	}
-	adb, err := devices.NewUbuntuDebugBridge()
-	var fastboot devices.Fastboot
-	if err != nil {
-		log.Fatal(err)
-	}
-	if args.Serial != "" {
-		adb.SetSerial(args.Serial)
-		fastboot.SetSerial(args.Serial)
-	}
-	if args.Device == "" {
-		if args.Bootstrap {
-			log.Print("Expecting the device to be in the bootloader... waiting")
-			args.Device, err = fastboot.GetDevice()
-		} else {
-			log.Print("Expecting the device to expose an adb interface...")
-			// TODO needs to work from recovery as well
-			//adb.WaitForDevice()
-			args.Device, err = adb.GetDevice()
-		}
-		if err != nil {
-			log.Fatalln("Cannot determine the device name:", err)
-		}
-	}
-	log.Printf("Device is |%s|", args.Device)
-	deviceChannel, err := channels.GetDeviceChannel(
-		args.Server, args.Channel, args.Device)
-	if err != nil {
-		log.Fatal(err)
-	}
-	var image ubuntuimage.Image
-	if args.Revision <= 0 {
-		image, err = deviceChannel.GetRelativeImage(args.Revision)
-	} else {
-		image, err = deviceChannel.GetImage(args.Revision)
-	}
-	if err != nil {
-		log.Fatal(err)
-	}
-	if args.ShowImage {
-		fmt.Printf("Description: %s\nVersion: %d\nChannel: %s\n", image.Description, image.Version, args.Channel)
-		for _, f := range image.Files {
-			f.MakeRelativeToServer(args.Server)
-			fmt.Printf("%d %s%s %d %s\n", f.Order, f.Server, f.Path, f.Size, f.Checksum)
-		}
-		return
-	}
-	log.Printf("Flashing version %d from %s channel and server %s to device %s",
-		image.Version, args.Channel, args.Server, args.Device)
-	if deviceChannel.Alias != "" {
-		log.Printf("%s is a channel alias to %s", deviceChannel.Alias, args.Channel)
-	}
-
-	// TODO use closures
-	signFiles := ubuntuimage.GetGPGFiles()
-	totalFiles := len(image.Files) + len(signFiles)
-	files := make(chan Files, totalFiles)
-	done := make(chan bool, totalFiles)
-	for i, file := range image.Files {
-		if tarballPath != "" && strings.HasPrefix(file.Path, "/pool/device") {
-			//change the file paths so they are correctly picked up by bitPusher later on
-			image.Files[i].Path = tarballPath
-			image.Files[i].Signature = tarballPath + ".asc"
-			useLocalTarball(image.Files[i], files)
-		} else {
-			go bitDownloader(file, files, args.Server, cacheDir)
-		}
-	}
-	for _, file := range signFiles {
-		go bitDownloader(file, files, args.Server, cacheDir)
-	}
-	if args.DownloadOnly {
-		for i := 0; i < totalFiles; i++ {
-			<-files
-		}
-		log.Printf("Downloaded files for version %d, channel %s, exiting without flashing as requested.\n", image.Version, args.Channel)
-		os.Exit(0)
-	}
-	if args.Bootstrap {
-		var downloadedFiles []Files
-		for i := 0; i < totalFiles; i++ {
-			downloadedFiles = append(downloadedFiles, <-files)
-		}
-		//Find the recovery image
-		var recovery string
-		for _, file := range downloadedFiles {
-			if strings.HasSuffix(file.FilePath, ".xz") {
-				fmt.Println(file.FilePath)
-				recovery, err = tryExtractRecovery(file.FilePath)
-				if err == nil {
-					break
-				}
-			}
-		}
-		if recovery == "" {
-			log.Fatal("Recovery image not found, cannot continue with bootstrap")
-		}
-		if err := fastboot.Flash("recovery", recovery); err != nil {
-			log.Fatal("Can't flash recovery image")
-		}
-		if err := fastboot.Format("cache"); err != nil {
-			log.Print("Cache formatting was not successful, flashing may fail, " +
-				"check your partitions on device")
-		}
-
-		if err := fastboot.BootImage(recovery); err != nil {
-			log.Fatal("Can't boot recovery image")
-		}
-		os.Remove(recovery)
-		if err := adb.WaitForRecovery(); err != nil {
-			log.Fatal(err)
-		}
-		// Resend all the files
-		for _, file := range downloadedFiles {
-			files <- file
-		}
-	}
-	go bitPusher(adb, files, done)
-	for i := 0; i < totalFiles; i++ {
-		<-done
-	}
-
-	var enableList []string
-	if args.DeveloperMode {
-		enableList = append(enableList, "developer_mode")
-	}
-	if args.Password != "" {
-		enableList = append(enableList, "default_password "+args.Password)
-	}
-
-	ubuntuCommands, err := ubuntuimage.GetUbuntuCommands(image.Files, cacheDir, args.Wipe, enableList)
-	if err != nil {
-		log.Fatal("Cannot create commands file")
-	}
-	log.Printf("Created ubuntu_command: %s", ubuntuCommands)
-
-	adb.Push(ubuntuCommands, "/cache/recovery/ubuntu_command")
-	defer func() {
-		if err == nil {
-			err = os.Remove(ubuntuCommands)
-			if err != nil {
-				log.Fatal(err)
-			}
-		}
-	}()
-
-	// either customize the flashing process by running a user provided script
-	// or reboot into recovery to let the standard upgrade script to run
-	if script != "" {
-		log.Printf("Preparing to run %s to finish the flashing process\n", script)
-		cmd := exec.Command(script)
-		cmd.Stdout = os.Stdout
-		err = cmd.Run()
-		if err != nil {
-			log.Fatal(err)
-		}
-
-	} else {
-		log.Print("Rebooting into recovery to flash")
-		adb.RebootRecovery()
-		err = adb.WaitForRecovery()
-		if err != nil {
-			log.Fatal(err)
-		}
-	}
-}
-
-// ensureExists touches a file. It can be used to create a dummy .asc file if none exists
-func ensureExists(path string) {
-	f, err := os.OpenFile(path, syscall.O_WRONLY|syscall.O_CREAT, 0666)
-	if err != nil {
-		log.Fatal("Cannot touch %s : %s", path, err)
-	}
-	f.Close()
-}
-
-type Files struct{ FilePath, SigPath string }
-
-// useLocalTarball adds a local file to the ones to be pushed
-func useLocalTarball(file ubuntuimage.File, files chan<- Files) {
-	ensureExists(file.Signature)
-	files <- Files{FilePath: file.Path, SigPath: file.Signature}
-}
-
-// bitDownloader downloads
-func bitDownloader(file ubuntuimage.File, files chan<- Files, server, downloadDir string) {
-	err := file.MakeRelativeToServer(server)
-	if err != nil {
-		log.Fatal(err)
-	}
-	err = file.Download(downloadDir)
-	if err != nil {
-		log.Fatal(err)
-	}
-	files <- Files{FilePath: filepath.Join(downloadDir, file.Path),
-		SigPath: filepath.Join(downloadDir, file.Signature)}
-}
-
-// bitPusher
-func bitPusher(adb devices.UbuntuDebugBridge, files <-chan Files, done chan<- bool) {
-	if err := adb.Ping(); err != nil {
-		log.Fatal("Target device cannot be reached over adb")
-	}
-	if _, err := adb.Shell("rm -rf /cache/recovery/*.xz /cache/recovery/*.xz.asc"); err != nil {
-		log.Fatal("Cannot cleanup /cache/recovery/ to ensure clean deployment", err)
-	}
-	freeSpace := "unknown"
-	dfCacheCmd := "df -h | grep /android/cache"
-	if free, err := adb.Shell(dfCacheCmd); err != nil {
-		log.Fatal("Unable to retrieve free space on target")
-	} else {
-		//Filesystem Size Used Avail Use% Mounted on
-		free := strings.Fields(free)
-		if len(free) > 3 {
-			freeSpace = free[3]
-		}
-	}
-	errMsg := "Cannot push %s to device: free space on /cache/recovery is %s"
-	for {
-		file := <-files
-		go func() {
-			log.Printf("Start pushing %s to device", file.FilePath)
-			err := adb.Push(file.FilePath, "/cache/recovery/")
-			if err != nil {
-				log.Fatalf(errMsg, file.SigPath, freeSpace)
-			}
-			err = adb.Push(file.SigPath, "/cache/recovery/")
-			if err != nil {
-				log.Fatalf(errMsg, file.SigPath, freeSpace)
-			}
-			log.Printf("Done pushing %s to device", file.FilePath)
-			done <- true
-		}()
-	}
-}
-
-func tryExtractRecovery(tarxz string) (recovery string, err error) {
-	f, err := os.Open(tarxz)
-	if err != nil {
-		log.Fatalf("Can't open %s in search for recovery", tarxz)
-	}
-	defer f.Close()
-	r := xzReader(f)
-	tempfile, err := ioutil.TempFile(os.TempDir(), "recoverytar")
-	if err != nil {
-		log.Fatal("Can't create tempfile to search for for recovery")
-	}
-	defer func() {
-		tempfile.Close()
-		os.Remove(tempfile.Name())
-	}()
-	n, err := io.Copy(tempfile, r)
-	if err != nil {
-		log.Fatalf("copied %d bytes with err: %v", n, err)
-	}
-	_, err = tempfile.Seek(0, 0)
-	if err != nil {
-		log.Fatal("Failed to rewind")
-	}
-	tr := tar.NewReader(tempfile)
-	var recoveryFile *os.File
-	//Going to return inside this loop, ugly, yeah
-	for {
-		hdr, err := tr.Next()
-		if err == io.EOF {
-			// end of tar archive
-			break
-		}
-		if err != nil {
-			log.Fatalln(err)
-		}
-		if strings.Contains(hdr.Name, "recovery.img") {
-			recoveryFile, err = ioutil.TempFile(os.TempDir(), "recovery")
-			defer recoveryFile.Close()
-			if err != nil {
-				log.Fatal(err)
-			}
-			if _, err := io.Copy(recoveryFile, tr); err != nil {
-				log.Fatal(err)
-			}
-			return recoveryFile.Name(), nil
-		}
-	}
-	return "", errors.New("Recovery Partition not found")
-}
-
-func xzReader(r io.Reader) io.ReadCloser {
-	rpipe, wpipe := io.Pipe()
-
-	cmd := exec.Command("xz", "--decompress", "--stdout")
-	cmd.Stdin = r
-	cmd.Stdout = wpipe
-
-	go func() {
-		err := cmd.Run()
-		wpipe.CloseWithError(err)
-	}()
-
-	return rpipe
+
+	if parser.Active == nil {
+		fmt.Println("DEPRECATED: Implicit 'touch' subcommand assumed")
+		touchCmd.Channel = globalArgs.Channel
+		if err := touchCmd.Execute(args); err != nil {
+			fmt.Println(err)
+			os.Exit(1)
+		}
+	}
 }

=== added file 'ubuntu-device-flash/query.go'
--- ubuntu-device-flash/query.go	1970-01-01 00:00:00 +0000
+++ ubuntu-device-flash/query.go	2014-10-09 12:18:37 +0000
@@ -0,0 +1,124 @@
+//
+// udbflash - Tool to download and flash devices with an Ubuntu Image based
+//            system
+//
+// Copyright (c) 2013 Canonical Ltd.
+//
+// Written by Sergio Schvezov <sergio.schvezov@canonical.com>
+//
+package main
+
+import (
+	"errors"
+	"fmt"
+
+	"launchpad.net/goget-ubuntu-touch/ubuntuimage"
+)
+
+// This program is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License version 3, as published
+// by the Free Software Foundation.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranties of
+// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
+// PURPOSE.  See the GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+func init() {
+	parser.AddCommand("query",
+		"Run queries against the image server",
+		"Choose from the list of query options to retrieve information from the server",
+		&queryCmd)
+
+	//queryCmd.AddCommand("list-channels", "List available channels", "", &channelCmd)
+}
+
+type QueryCmd struct {
+	ListChannels bool   `long:"list-channels" description:"List available channels"`
+	ListImages   bool   `long:"list-images" description:"List available images for a channel"`
+	ShowImage    bool   `long:"show-image" description:"Show information for an image in the given channel"`
+	Channel      string `long:"channel" description:"Specify an alternate channel"`
+	Device       string `long:"device" description:"Specify the device to use as a base for querying" required:"true"`
+}
+
+var queryCmd QueryCmd
+
+func (queryCmd *QueryCmd) Execute(args []string) error {
+	if queryCmd.ListChannels {
+		return queryCmd.printChannelList()
+	}
+
+	if queryCmd.ShowImage {
+		return queryCmd.printImageInformation()
+	}
+
+	if queryCmd.ListImages {
+		return queryCmd.printImageList()
+	}
+
+	return errors.New("A query option is requrired")
+}
+
+func (queryCmd *QueryCmd) printImageList() error {
+	channels, err := ubuntuimage.NewChannels(globalArgs.Server)
+	if err != nil {
+		return err
+	}
+
+	deviceChannel, err := channels.GetDeviceChannel(
+		globalArgs.Server, queryCmd.Channel, queryCmd.Device)
+	if err != nil {
+		return err
+	}
+
+	return deviceChannel.ListImageVersions()
+}
+
+func (queryCmd *QueryCmd) printChannelList() error {
+	channels, err := ubuntuimage.NewChannels(globalArgs.Server)
+	if err != nil {
+		return err
+	}
+
+	for k, v := range channels {
+		if v.Alias != "" {
+			fmt.Printf("%s (alias to %s)\n", k, v.Alias)
+		} else {
+			fmt.Println(k)
+		}
+	}
+
+	return nil
+}
+
+func (queryCmd *QueryCmd) printImageInformation() error {
+	channels, err := ubuntuimage.NewChannels(globalArgs.Server)
+	if err != nil {
+		return err
+	}
+
+	if queryCmd.Channel == "" {
+		return errors.New("channel required")
+	}
+
+	deviceChannel, err := channels.GetDeviceChannel(globalArgs.Server, queryCmd.Channel, queryCmd.Device)
+	if err != nil {
+		return err
+	}
+
+	image, err := getImage(deviceChannel)
+	if err != nil {
+		return err
+	}
+
+	fmt.Printf("Device: %s\nDescription: %s\nVersion: %d\nChannel: %s\nFiles:\n", queryCmd.Device, image.Description, image.Version, queryCmd.Channel)
+	for _, f := range image.Files {
+		f.MakeRelativeToServer(globalArgs.Server)
+		fmt.Printf(" %d %s%s %d %s\n", f.Order, f.Server, f.Path, f.Size, f.Checksum)
+	}
+
+	return nil
+}

=== added file 'ubuntu-device-flash/touch.go'
--- ubuntu-device-flash/touch.go	1970-01-01 00:00:00 +0000
+++ ubuntu-device-flash/touch.go	2014-10-09 12:18:37 +0000
@@ -0,0 +1,384 @@
+//
+// udbflash - Tool to download and flash devices with an Ubuntu Image based
+//            system
+//
+// Copyright (c) 2013-2014 Canonical Ltd.
+//
+// Written by Sergio Schvezov <sergio.schvezov@canonical.com>
+//
+package main
+
+import (
+	"archive/tar"
+	"errors"
+	"fmt"
+	"io"
+	"io/ioutil"
+	"log"
+	"os"
+	"os/exec"
+	"path/filepath"
+	"strings"
+	"syscall"
+
+	"launchpad.net/goget-ubuntu-touch/devices"
+	"launchpad.net/goget-ubuntu-touch/ubuntuimage"
+)
+
+func init() {
+	parser.AddCommand("touch",
+		"Flashes ubuntu touch images",
+		"",
+		&touchCmd)
+}
+
+type TouchCmd struct {
+	Bootstrap     bool   `long:"bootstrap" description:"bootstrap the system, do this from the bootloader"`
+	Wipe          bool   `long:"wipe" description:"Clear all data after flashing"`
+	Serial        string `long:"serial" description:"Serial of the device to operate"`
+	DeveloperMode bool   `long:"developer-mode" description:"Enables developer mode after the factory reset"`
+	DeviceTarball string `long:"device-tarball" description:"Specify a local device tarball to override the one from the server (using official Ubuntu images with custom device tarballs)"`
+	RunScript     string `long:"run-script" description:"Run a script given by path to finish the flashing process, instead of rebooting to recovery (mostly used during development to work around quirky or incomplete recovery images)"`
+	Password      string `long:"password" description:"This sets up the default password for the phablet user. This option is meant for CI and not general use"`
+	Channel       string `long:"channel" description:"Specify the channel to use" default:"ubuntu-touch/stable"`
+	Device        string `long:"device" description:"Specify the device to flash"`
+	fastboot      devices.Fastboot
+	adb           devices.UbuntuDebugBridge
+}
+
+var touchCmd TouchCmd
+
+func (touchCmd *TouchCmd) Execute(args []string) error {
+	if globalArgs.TLSSkipVerify {
+		ubuntuimage.TLSSkipVerify()
+	}
+
+	script := touchCmd.RunScript
+	if script != "" {
+		if p, err := filepath.Abs(script); err != nil {
+			log.Fatal("Run script not found:", err)
+		} else {
+			script = p
+		}
+
+		fi, err := os.Lstat(script)
+		if err != nil {
+			log.Fatal(err)
+		}
+		if fi.Mode()&0100 == 0 || !fi.Mode().IsRegular() {
+			log.Fatalf("The script %s passed via --run-script is not executable", script)
+		}
+	}
+
+	if touchCmd.Bootstrap {
+		touchCmd.Wipe = true
+	}
+
+	if touchCmd.Password != "" && !touchCmd.Wipe && !touchCmd.DeveloperMode {
+		log.Fatal("Default password setup requires --developer-mode, and --wipe or --bootstrap")
+	}
+
+	tarballPath := touchCmd.DeviceTarball
+	if tarballPath != "" {
+		if p, err := filepath.Abs(tarballPath); err != nil {
+			log.Fatal("Device tarball not found", err)
+		} else {
+			tarballPath = p
+		}
+		fi, err := os.Lstat(tarballPath)
+		if err != nil {
+			log.Fatal(err)
+		}
+		if !fi.Mode().IsRegular() {
+			log.Fatalf("The file %s passed via --device-tarball is not a regular file\n", tarballPath)
+		}
+	}
+
+	channels, err := ubuntuimage.NewChannels(globalArgs.Server)
+	if err != nil {
+		return err
+	}
+
+	if globalArgs.CleanCache {
+		log.Print("Cleaning prevously downloaded content")
+		return os.RemoveAll(cacheDir)
+	}
+
+	if err := touchCmd.setupDevice(); err != nil {
+		return err
+	}
+	log.Printf("Device is |%s|", touchCmd.Device)
+
+	deviceChannel, err := channels.GetDeviceChannel(globalArgs.Server, touchCmd.Channel, touchCmd.Device)
+	if err != nil {
+		return err
+	}
+
+	image, err := getImage(deviceChannel)
+	if err != nil {
+		return err
+	}
+
+	log.Printf("Flashing version %d from %s channel and server %s to device %s",
+		image.Version, touchCmd.Channel, globalArgs.Server, touchCmd.Device)
+
+	// TODO use closures
+	signFiles := ubuntuimage.GetGPGFiles()
+	totalFiles := len(image.Files) + len(signFiles)
+	files := make(chan Files, totalFiles)
+	done := make(chan bool, totalFiles)
+
+	for i, file := range image.Files {
+		if tarballPath != "" && strings.HasPrefix(file.Path, "/pool/device") {
+			//change the file paths so they are correctly picked up by bitPusher later on
+			image.Files[i].Path = tarballPath
+			image.Files[i].Signature = tarballPath + ".asc"
+			useLocalTarball(image.Files[i], files)
+		} else {
+			go bitDownloader(file, files, globalArgs.Server, cacheDir)
+		}
+	}
+
+	for _, file := range signFiles {
+		go bitDownloader(file, files, globalArgs.Server, cacheDir)
+	}
+
+	if globalArgs.DownloadOnly {
+		for i := 0; i < totalFiles; i++ {
+			<-files
+		}
+		log.Printf("Downloaded files for version %d, channel %s, exiting without flashing as requested.\n", image.Version, touchCmd.Channel)
+		return nil
+	}
+
+	if touchCmd.Bootstrap {
+		var downloadedFiles []Files
+		for i := 0; i < totalFiles; i++ {
+			downloadedFiles = append(downloadedFiles, <-files)
+		}
+		//Find the recovery image
+		var recovery string
+		for _, file := range downloadedFiles {
+			if strings.HasSuffix(file.FilePath, ".xz") {
+				fmt.Println(file.FilePath)
+				recovery, err = tryExtractRecovery(file.FilePath)
+				if err == nil {
+					break
+				}
+			}
+		}
+		if recovery == "" {
+			return errors.New("recovery image not found, cannot continue with bootstrap")
+		}
+		if err := touchCmd.fastboot.Flash("recovery", recovery); err != nil {
+			return errors.New("can't flash recovery image")
+		}
+		if err := touchCmd.fastboot.Format("cache"); err != nil {
+			log.Print("Cache formatting was not successful, flashing may fail, " +
+				"check your partitions on device")
+		}
+
+		if err := touchCmd.fastboot.BootImage(recovery); err != nil {
+			return errors.New("Can't boot recovery image")
+		}
+		os.Remove(recovery)
+		if err := touchCmd.adb.WaitForRecovery(); err != nil {
+			return err
+		}
+		// Resend all the files
+		for _, file := range downloadedFiles {
+			files <- file
+		}
+	}
+	go bitPusher(touchCmd.adb, files, done)
+	for i := 0; i < totalFiles; i++ {
+		<-done
+	}
+
+	var enableList []string
+	if touchCmd.DeveloperMode {
+		enableList = append(enableList, "developer_mode")
+	}
+	if touchCmd.Password != "" {
+		enableList = append(enableList, "default_password "+touchCmd.Password)
+	}
+
+	ubuntuCommands, err := ubuntuimage.GetUbuntuCommands(image.Files, cacheDir, touchCmd.Wipe, enableList)
+	if err != nil {
+		return errors.New("cannot create commands file")
+	}
+	log.Printf("Created ubuntu_command: %s", ubuntuCommands)
+
+	touchCmd.adb.Push(ubuntuCommands, "/cache/recovery/ubuntu_command")
+	defer os.Remove(ubuntuCommands)
+
+	// either customize the flashing process by running a user provided script
+	// or reboot into recovery to let the standard upgrade script to run
+	if script != "" {
+		log.Printf("Preparing to run %s to finish the flashing process\n", script)
+		cmd := exec.Command(script)
+		cmd.Stdout = os.Stdout
+		err = cmd.Run()
+		if err != nil {
+			return err
+		}
+
+	} else {
+		log.Print("Rebooting into recovery to flash")
+		touchCmd.adb.RebootRecovery()
+		err = touchCmd.adb.WaitForRecovery()
+		if err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+func (touchCmd *TouchCmd) setupDevice() (err error) {
+	if adb, err := devices.NewUbuntuDebugBridge(); err == nil {
+		touchCmd.adb = adb
+	} else {
+		return err
+	}
+
+	if touchCmd.Serial != "" {
+		touchCmd.adb.SetSerial(touchCmd.Serial)
+		touchCmd.fastboot.SetSerial(touchCmd.Serial)
+	}
+
+	if touchCmd.Device == "" {
+		if touchCmd.Bootstrap {
+			log.Print("Expecting the device to be in the bootloader... waiting")
+			touchCmd.Device, err = touchCmd.fastboot.GetDevice()
+			return err
+		} else {
+			log.Print("Expecting the device to expose an adb interface...")
+			// TODO needs to work from recovery as well
+			//adb.WaitForDevice()
+			touchCmd.Device, err = touchCmd.adb.GetDevice()
+			if err != nil {
+				return errors.New("device cannot be detected over adb")
+			}
+		}
+	}
+
+	return nil
+}
+
+// ensureExists touches a file. It can be used to create a dummy .asc file if none exists
+func ensureExists(path string) {
+	f, err := os.OpenFile(path, syscall.O_WRONLY|syscall.O_CREAT, 0666)
+	if err != nil {
+		log.Fatal("Cannot touch %s : %s", path, err)
+	}
+	f.Close()
+}
+
+// useLocalTarball adds a local file to the ones to be pushed
+func useLocalTarball(file ubuntuimage.File, files chan<- Files) {
+	ensureExists(file.Signature)
+	files <- Files{FilePath: file.Path, SigPath: file.Signature}
+}
+
+// bitPusher
+func bitPusher(adb devices.UbuntuDebugBridge, files <-chan Files, done chan<- bool) {
+	if err := adb.Ping(); err != nil {
+		log.Fatal("Target device cannot be reached over adb")
+	}
+	if _, err := adb.Shell("rm -rf /cache/recovery/*.xz /cache/recovery/*.xz.asc"); err != nil {
+		log.Fatal("Cannot cleanup /cache/recovery/ to ensure clean deployment", err)
+	}
+	freeSpace := "unknown"
+	dfCacheCmd := "df -h | grep /android/cache"
+	if free, err := adb.Shell(dfCacheCmd); err != nil {
+		log.Fatal("Unable to retrieve free space on target")
+	} else {
+		//Filesystem Size Used Avail Use% Mounted on
+		free := strings.Fields(free)
+		if len(free) > 3 {
+			freeSpace = free[3]
+		}
+	}
+	errMsg := "Cannot push %s to device: free space on /cache/recovery is %s"
+	for {
+		file := <-files
+		go func() {
+			log.Printf("Start pushing %s to device", file.FilePath)
+			err := adb.Push(file.FilePath, "/cache/recovery/")
+			if err != nil {
+				log.Fatalf(errMsg, file.SigPath, freeSpace)
+			}
+			err = adb.Push(file.SigPath, "/cache/recovery/")
+			if err != nil {
+				log.Fatalf(errMsg, file.SigPath, freeSpace)
+			}
+			log.Printf("Done pushing %s to device", file.FilePath)
+			done <- true
+		}()
+	}
+}
+
+func tryExtractRecovery(tarxz string) (recovery string, err error) {
+	f, err := os.Open(tarxz)
+	if err != nil {
+		log.Fatalf("Can't open %s in search for recovery", tarxz)
+	}
+	defer f.Close()
+	r := xzReader(f)
+	tempfile, err := ioutil.TempFile(os.TempDir(), "recoverytar")
+	if err != nil {
+		log.Fatal("Can't create tempfile to search for for recovery")
+	}
+	defer func() {
+		tempfile.Close()
+		os.Remove(tempfile.Name())
+	}()
+	n, err := io.Copy(tempfile, r)
+	if err != nil {
+		log.Fatalf("copied %d bytes with err: %v", n, err)
+	}
+	_, err = tempfile.Seek(0, 0)
+	if err != nil {
+		log.Fatal("Failed to rewind")
+	}
+	tr := tar.NewReader(tempfile)
+	var recoveryFile *os.File
+	//Going to return inside this loop, ugly, yeah
+	for {
+		hdr, err := tr.Next()
+		if err == io.EOF {
+			// end of tar archive
+			break
+		}
+		if err != nil {
+			log.Fatalln(err)
+		}
+		if strings.Contains(hdr.Name, "recovery.img") {
+			recoveryFile, err = ioutil.TempFile(os.TempDir(), "recovery")
+			defer recoveryFile.Close()
+			if err != nil {
+				log.Fatal(err)
+			}
+			if _, err := io.Copy(recoveryFile, tr); err != nil {
+				log.Fatal(err)
+			}
+			return recoveryFile.Name(), nil
+		}
+	}
+	return "", errors.New("Recovery Partition not found")
+}
+
+func xzReader(r io.Reader) io.ReadCloser {
+	rpipe, wpipe := io.Pipe()
+
+	cmd := exec.Command("xz", "--decompress", "--stdout")
+	cmd.Stdin = r
+	cmd.Stdout = wpipe
+
+	go func() {
+		err := cmd.Run()
+		wpipe.CloseWithError(err)
+	}()
+
+	return rpipe
+}

=== modified file 'ubuntu-emulator/create.go'
--- ubuntu-emulator/create.go	2014-09-09 20:22:48 +0000
+++ ubuntu-emulator/create.go	2014-10-09 12:18:37 +0000
@@ -27,8 +27,8 @@
 	"runtime"
 	"syscall"
 
-	"launchpad.net/goget-ubuntu-touch/ubuntu-emulator/diskimage"
-	"launchpad.net/goget-ubuntu-touch/ubuntu-emulator/sysutils"
+	"launchpad.net/goget-ubuntu-touch/diskimage"
+	"launchpad.net/goget-ubuntu-touch/sysutils"
 	"launchpad.net/goget-ubuntu-touch/ubuntuimage"
 )
 
@@ -161,10 +161,10 @@
 		}
 	}
 
-	if err = sysutils.WriteStamp(dataDir, image); err != nil {
+	if err = writeStamp(dataDir, image); err != nil {
 		return err
 	}
-	if err = sysutils.WriteDeviceStamp(dataDir, createCmd.Arch); err != nil {
+	if err = writeDeviceStamp(dataDir, createCmd.Arch); err != nil {
 		return err
 	}
 

=== modified file 'ubuntu-emulator/list.go'
--- ubuntu-emulator/list.go	2014-02-19 14:26:01 +0000
+++ ubuntu-emulator/list.go	2014-10-09 12:18:37 +0000
@@ -22,7 +22,6 @@
 import (
 	"fmt"
 	"io/ioutil"
-	"launchpad.net/goget-ubuntu-touch/ubuntu-emulator/sysutils"
 	"path/filepath"
 )
 
@@ -49,7 +48,7 @@
 		if !entry.IsDir() {
 			continue
 		}
-		if image, err := sysutils.ReadStamp(filepath.Join(dataDir, entry.Name())); err == nil {
+		if image, err := readStamp(filepath.Join(dataDir, entry.Name())); err == nil {
 			fmt.Printf("%s\t%s\n", entry.Name(), image.Description)
 		} else {
 			fmt.Println(entry.Name())

=== modified file 'ubuntu-emulator/run.go'
--- ubuntu-emulator/run.go	2014-08-07 02:51:32 +0000
+++ ubuntu-emulator/run.go	2014-10-09 12:18:37 +0000
@@ -26,8 +26,6 @@
 	"os/exec"
 	"path"
 	"path/filepath"
-
-	"launchpad.net/goget-ubuntu-touch/ubuntu-emulator/sysutils"
 )
 
 type RunCmd struct {
@@ -85,7 +83,7 @@
 		return err
 	}
 
-	device, err := sysutils.ReadDeviceStamp(dataDir)
+	device, err := readDeviceStamp(dataDir)
 	if err != nil {
 		return err
 	}

=== modified file 'ubuntu-emulator/snapshot.go'
--- ubuntu-emulator/snapshot.go	2014-01-07 15:19:28 +0000
+++ ubuntu-emulator/snapshot.go	2014-10-09 12:18:37 +0000
@@ -22,8 +22,9 @@
 import (
 	"errors"
 	"fmt"
-	"launchpad.net/goget-ubuntu-touch/ubuntu-emulator/diskimage"
 	"path/filepath"
+
+	"launchpad.net/goget-ubuntu-touch/diskimage"
 )
 
 type SnapshotCmd struct {

=== renamed file 'ubuntu-emulator/sysutils/stamp.go' => 'ubuntu-emulator/stamp.go'
--- ubuntu-emulator/sysutils/stamp.go	2014-04-23 18:09:19 +0000
+++ ubuntu-emulator/stamp.go	2014-10-09 12:18:37 +0000
@@ -5,7 +5,7 @@
 //
 // Written by Sergio Schvezov <sergio.schvezov@canonical.com>
 //
-package sysutils
+package main
 
 // This program is free software: you can redistribute it and/or modify it
 // under the terms of the GNU General Public License version 3, as published
@@ -23,17 +23,18 @@
 	"bufio"
 	"encoding/json"
 	"io/ioutil"
-	"launchpad.net/goget-ubuntu-touch/ubuntuimage"
 	"os"
 	"path/filepath"
+
+	"launchpad.net/goget-ubuntu-touch/ubuntuimage"
 )
 
-func WriteDeviceStamp(dataDir, device string) (err error) {
+func writeDeviceStamp(dataDir, device string) (err error) {
 	path := filepath.Join(dataDir, ".device")
 	return ioutil.WriteFile(path, []byte(device), 0600)
 }
 
-func ReadDeviceStamp(dataDir string) (string, error) {
+func readDeviceStamp(dataDir string) (string, error) {
 	path := filepath.Join(dataDir, ".device")
 	device, err := ioutil.ReadFile(path)
 	if err != nil {
@@ -42,7 +43,7 @@
 	return string(device), nil
 }
 
-func WriteStamp(dataDir string, image ubuntuimage.Image) (err error) {
+func writeStamp(dataDir string, image ubuntuimage.Image) (err error) {
 	path := filepath.Join(dataDir, ".stamp")
 	file, err := os.Create(path)
 	if err != nil {
@@ -64,7 +65,7 @@
 	return nil
 }
 
-func ReadStamp(dataDir string) (image ubuntuimage.Image, err error) {
+func readStamp(dataDir string) (image ubuntuimage.Image, err error) {
 	path := filepath.Join(dataDir, ".stamp")
 	file, err := os.Open(path)
 	if err != nil {

